﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;

namespace DymWebForm.Common
{
    internal static class TypeParser<T> where T : struct
    {
        private delegate bool DTryParse(string value, out T result);

        private static readonly DTryParse TryParse;

        static TypeParser()
        {
            var type = typeof(T);

            if (type.Name == "Nullable`1")
                throw new NotSupportedException("TypeParser not support Nullable type");

            var mTryParse = type
                .GetMethods(
                    BindingFlags.Static
                    | BindingFlags.Public
                    | BindingFlags.DeclaredOnly
                    | BindingFlags.InvokeMethod
                ).FirstOrDefault(p =>
                    p.ReturnType == typeof(bool)
                    && p.GetParameters().Length == 2
                    && p.GetParameters()[0].ParameterType == typeof(string)
                    && p.GetParameters()[1].IsOut
                    && p.GetParameters()[1].ParameterType.GetElementType() == type
                );

            if (mTryParse == null)
                throw new NotSupportedException("TypeParser can no found `bool TypParse(String,out " + type.Name + ")` metohd");

            TryParse = (DTryParse)Delegate.CreateDelegate(typeof(DTryParse), mTryParse);
        }

        public static T Convert(object value)
        {
            if (value == null || value == DBNull.Value)
                return default(T);

            T result;
            TryParse(value.ToString(), out result);
            return result;
        }

        public static T? ConvertNullable(object value)
        {
            if (value == null || value == DBNull.Value)
                return null;

            T result;
            if (TryParse(value.ToString(), out result))
                return result;
            return null;
        }
    }
}
